unit sCustomComboBox;
(************************************************************************
sStyle components package 
Copyright (c) 2002 Serge Goncharov. All rights reserved.
                            Ukraine, Odessa
                            http://www.geocities.com/sstylish2002/
                            Serge@HyBce.com
 Version:	1.2, 24-02-2002
*************************************************************************)

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, StrUtils, sStyleUtil, ToolEdit, CurrEdit, ComCtrls,
  ToolWin, ExtCtrls, RXCtrls, sConst, sUtils, sGraphUtils;

Type

  TsCustomComboBox = class(TCustomComboBox)
  private
    FAlignment : TAlignment;
    FColorArrow: TColor;
    FColorButton: TColor;

    FButtonWidth: Integer;

    procedure CNCommand (var Message: TWMCommand); message CN_COMMAND;
    function GetButtonRect: TRect;
    function GetClientRect: TRect;
    procedure MEnter(var Msg:TMessage); message CM_ENTER;
    procedure MExit(var Msg:TMessage); message CM_EXIT;
    procedure PaintButton;
    procedure RedrawBorders;
    procedure SetColors (Index: Integer; Value: TColor);
    procedure WMPaint (var Message: TWMPaint); message WM_PAINT;
    procedure SetAlignment(const Value: TAlignment);
  protected

    FActive : boolean;
    FCharsInCode : integer;
    FSQL : TStrings;
    FsStyle : TsStyle;
    FUseItemIndex : boolean;
    BorderStyle : TFormBorderStyle;

    procedure SetSQL(Value: TStrings);
    procedure SetActive(Value : boolean);

    procedure WndProc (var Message: TMessage); override;
    { Private declarations }
  public
    FChildHandle: HWND;
    FDefListProc: Pointer;
    FListHandle: HWND;
    FListInstance: Pointer;
    constructor Create(AOwner:TComponent); override;
    destructor Destroy; override;
    function Generate : integer; virtual;
    function GetCurrentKod : integer;
    function GetCurrentName : string;
    function IndexOfKod(s : string) : integer;
    procedure Invalidate; override;
    procedure InvalidateSelection;
    procedure WriteText;
    property Color;

    { Public declarations }
  published
    property Alignment : TAlignment read FAlignment write SetAlignment;
    property ColorArrow: TColor index 0 read FColorArrow write SetColors default clBlack;
    property ColorButton: TColor index 1 read FColorButton write SetColors default $00C5D6D9;

    property UseItemIndex : boolean read FUseItemIndex write FUseItemIndex;
    property ButtonWidth : integer read FButtonWidth write FButtonWidth;
    property ItemIndex default -1;
    property Enabled;
    property Font;
    property Hint;
    property ItemHeight;
    property Items;
    property MaxLength;
    property DropDownCount;
    property ParentFont;
    property ParentShowHint;
    property PopupMenu;
    property ShowHint;
    property Sorted;
    property Style;
    property TabOrder;
    property Text;
    property Visible;
    property CharCase;
    property sStyle:TsStyle read FsStyle write FsStyle;
    property Active:boolean read FActive write SetActive;
    property SQL:TStrings read FSQL write SetSQL;
    property CharsInCode:integer read FCharsInCode write FCharsInCode;

    property OnChange;
    property OnClick;
    property OnContextPopup;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnDrawItem;
    property OnEndDock;
    property OnEndDrag;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property OnMeasureItem;
    property OnStartDock;
    property OnStartDrag;
    property OnEnter;
    property OnExit;
    { Published declarations }
  end;

implementation

constructor TsCustomComboBox.Create(AOwner:TComponent);
begin
  inherited Create(AOwner);
  ControlStyle := ControlStyle - [csFixedHeight, csFramed] + [csOpaque];
  TControlCanvas(Canvas).Control := self;

  ParentColor := False;
  sStyle := TsStyle.Create(Self);
  sStyle.COC := COC_TsCustomComboBox;

  OnKeyDown := sStyle.OnKeyDown;
  FButtonWidth := 14;
  FDefListProc := nil;
  ItemHeight := 13;
  FColorArrow := clMenuText;
  if sStyle.Flat then begin
    FColorButton := sStyle.Painting.ColorInActive;
  end
  else begin
    FColorButton := clBtnFace;
  end;

  FSQL := TStringList.Create;
  CharsInCode := 0;
  UseItemIndex := False;
  DropDownCount := 16;
end;

destructor TsCustomComboBox.Destroy;
begin
  SQL.Free;
  sStyle.Free;
  inherited Destroy;
end;

procedure TsCustomComboBox.SetSQL(Value: TStrings);
begin
  SQL.Assign(Value);
end;

function TsCustomComboBox.GetCurrentKod : integer;
begin
  Result := -1;
  if Active and (CharsInCode>0) and (Text <> '') then begin
    try
      Result := StrToInt(copy(Text, 1, CharsInCode));
    except
    end;
  end;
end;

function TsCustomComboBox.IndexOfKod(s : string) : integer;
var
  i : integer;
  c1, c2 : string;
begin
  Result := -1;
  for i := 0 to Items.Count - 1 do begin
    c1 := AddChar('0', s, CharsInCode);
    c2 := ExtractWord(1, Items[i], [' ', '-']);
    if c1 = c2 then begin
      Result := i;
      Break;
    end;
  end;
end;

function TsCustomComboBox.GetCurrentName : string;
begin
  if CharsInCode > 0 then begin
    Result := copy(Text, CharsInCode + 4, Length(Text) - CharsInCode - 3);
  end
  else begin
    Result := Text;
  end;
end;

procedure TsCustomComboBox.SetActive(Value : boolean);
begin
  if Value then begin
    FActive := (Generate > 0);
  end
  else begin
    FActive := False;
  end;
end;

procedure TsCustomComboBox.WMPaint(var Message: TWMPaint);
var
  R, bR: TRect;
  DC: HDC;
  PS: TPaintStruct;
  WinBrush: HBRUSH;
begin
  if sStyle.Flat then begin
    DC := BeginPaint(Handle, PS);
    try
      R := PS.rcPaint;
      R.Right := iffi(R.Right > Width - FButtonWidth - 4, Width - FButtonWidth - 4, R.Right);
      WinBrush := CreateSolidBrush(ColorToRGB(sStyle.GetActiveColor));

      FillRect(DC, R, WinBrush);
      bR := GetButtonRect;
      if IntersectRect(bR, bR, PS.rcPaint) then PaintButton;
      ExcludeClipRect(DC, ClientWidth - FButtonWidth - 2, 0, ClientWidth, ClientHeight);
//      PaintWindow(DC);
      if (Style = csDropDown) and DroppedDown then begin
        R := ClientRect;
        InflateRect(R, -2, -2);
        R.Right := Width - FButtonWidth - 8;
        Canvas.Brush.Color := sStyle.GetActiveColor;
        Canvas.FrameRect(R);
      end
      else if Style <> csDropDown then InvalidateSelection;
    finally
      EndPaint(Handle, PS);
    end;
    RedrawBorders;
    DeleteObject(WinBrush);
    Message.Result := 0;
  end
  else inherited;
end;

//   
function TsCustomComboBox.GetButtonRect: TRect;
begin
  Result := ClientRect;
  Inc(Result.Left, ClientWidth - FButtonWidth);
  Inc(Result.Right, -1);
end;

//  
procedure TsCustomComboBox.PaintButton;
var
  R: TRect;
  x, y, i: Integer;
  procedure PolyDown;
  begin
    canvas.Polygon([Point(x + 3, y + 3), Point(x + 9, y + 3), Point(x + 6, y)])
  end;
  procedure PolyUp;
  begin
    canvas.Polygon([Point(x + 3, y), Point(x + 9, y), Point(x + 6, y + 3)]);
  end;
begin
  R := GetButtonRect;
  InflateRect(R, 1, 0);

  Canvas.Brush.Color := FColorButton;
  Canvas.FillRect(R);
  Canvas.Brush.Color := sStyle.Painting.ColorBorderTop;

  i := 1;
  x := (R.Right - R.Left) div 2 - 6 + R.Left;
  y := (R.Bottom - R.Top) div 2 - 1 + R.Top;
  DrawRectangleOnDC(Canvas.Handle, R, sStyle.Painting.ColorBorderTop, sStyle.Painting.ColorBorderBottom, i);

  if Enabled then begin
    canvas.Brush.Color := FColorArrow;
    canvas.Pen.Color := FColorArrow;
    if DroppedDown then PolyDown else PolyUp;
  end
  else begin
    canvas.Brush.Color := clWhite;
    canvas.Pen.Color := clWhite;
    Inc(x); Inc(y);
    if DroppedDown then PolyDown else PolyUp;

//    PolyDown;
    Dec(x); Dec(y);
    canvas.Brush.Color := clGray;
    canvas.Pen.Color := clGray;
    if DroppedDown then PolyDown else PolyUp;
//    PolyDown;
  end;
  ExcludeClipRect(Canvas.Handle, ClientWidth - FButtonWidth, 0, ClientWidth, ClientHeight);
end;

procedure TsCustomComboBox.CNCommand(var Message: TWMCommand);
var
  DC: HDC;
  WinBrush: HBRUSH;
  R: TRect;
begin
  inherited;
  if sStyle.Flat then begin
    Case Message.NotifyCode of
      CBN_SELCHANGE, CBN_SELENDOK, CBN_DROPDOWN: begin
        if not (Style in [csSimple, csDropDown]) then begin
          InvalidateSelection;
        end
        else if Message.NotifyCode = CBN_DROPDOWN then begin
          DC := GetWindowDC(Handle);
          WinBrush := CreateSolidBrush(ColorToRGB(sStyle.GetActiveColor));

          R := ClientRect;
          InflateRect(R, -2, -2);
          R.Top := R.Top + GetFontHeight(Font.Handle) - 5;
          FillRect(DC, R, WinBrush);

          R := ClientRect;
          R.Left := R.Left + 2 + self.Canvas.TextWidth(Text);
          FillRect(DC, R, WinBrush);

          ReleaseDC(Handle, DC);
          DeleteObject(WinBrush);
        end;
      end;
      CBN_CLOSEUP: begin
        R := GetButtonRect;
        R.Right := R.Left - 2;
        InvalidateRect(Handle, @R, FALSE);
      end;
    end;
  end;
end;

procedure TsCustomComboBox.InvalidateSelection;
var
  DC: HDC;
  R: TRect;
  WinBrush: HBRUSH;
begin
  if not sStyle.FDrawingStop then begin
    if sStyle.Flat then begin
      if (GetFocus = Handle) and not DroppedDown then begin
        R := ClientRect;
        DC := GetWindowDC(Handle);
        inc(R.Right, - FButtonWidth - 1);

        WinBrush := CreateSolidBrush(ColorToRGB(sStyle.Painting.ColorInActive));
        InflateRect(R, -2, -2);
        FrameRect(DC, R, WinBrush);

        ReleaseDC(Handle, DC);
        DeleteObject(WinBrush);
      end;
    end;
  end;
end;

procedure TsCustomComboBox.RedrawBorders;
begin
  Color := sStyle.GetActiveColor;
  WriteText;
  sStyle.RedrawBorder;
  if (Style <> csSimple) then PaintButton;
end;

procedure TsCustomComboBox.SetColors(Index: Integer; Value: TColor);
begin
  case Index of
    0: if FColorArrow <> Value then begin
      FColorArrow := Value;
      Invalidate;
    end;
    1: if FColorButton <> Value then begin
      FColorButton := Value;
      Invalidate;
    end;
  end;
end;

procedure TsCustomComboBox.WndProc(var Message: TMessage);
begin
  sStyle.WndProc(Message);
  if Message.Result < 1 then inherited;
end;

procedure TsCustomComboBox.Invalidate;
begin
  Color := sStyle.GetActiveColor;
  if sStyle.Flat then begin
    if Assigned(Parent) and not sStyle.FDrawingStop then RedrawBorders;
  end;
  Inherited;
end;


procedure TsCustomComboBox.MEnter(var Msg: TMessage);
begin
  sStyle.FFocused := True;
  sStyle.ChangeFocus(True);
  invalidate;
  inherited;
end;

procedure TsCustomComboBox.MExit(var Msg: TMessage);
begin
  sStyle.FFocused := False;
  sStyle.ChangeFocus(False);
  invalidate;
  inherited;
end;

function TsCustomComboBox.Generate: integer;
begin
  Result := 0;
end;

procedure TsCustomComboBox.WriteText;
var
  Flags: Longint;
  R : TRect;
begin
  Canvas.Font := Font;
  R := GetClientRect;

  case Alignment of
    taLeftJustify : begin
      Flags := DT_LEFT;
    end;
    taRightJustify : begin
      Flags := DT_RIGHT;
    end;
    taCenter : begin
      Flags := DT_CENTER;
    end
  end;
  Flags := Flags or DT_EXPANDTABS or DT_VCENTER or DT_SINGLELINE;
  Flags := DrawTextBiDiModeFlags(Flags);
  sStyle.PaintWindow(Canvas, R.Left, R.Top, R.Right - R.Left, R.Bottom - R.Top, Text, Enabled, Flags);
end;

procedure TsCustomComboBox.SetAlignment(const Value: TAlignment);
begin
  if FAlignment <> Value then begin
    FAlignment := Value;
    Invalidate;
  end;;
end;

function TsCustomComboBox.GetClientRect: TRect;
begin
  Result := ClientRect;
  Dec(Result.Right, FButtonWidth);
  InflateRect(Result, - sStyle.FBorderWidth, - sStyle.FBorderWidth);
end;

end.
