unit sCustomMenuManager;
(************************************************************************
sStyle components package 
Copyright (c) 2002 Serge Goncharov. All rights reserved.
                            Ukraine, Odessa
                            http://www.geocities.com/sstylish2002/
                            Serge@HyBce.com
 Version:	1.2, 24-02-2002
*************************************************************************)

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs, sConst,
  sStyleUtil, sVclUtils, Menus, sUtils, ExtCtrls, sHintManager;

type
  TsCustomMenuManager = class(TComponent)
  private
    { Private declarations }
    FsStyle : TsPassivePaintStyle;
    FMnuList : TList;
    FActive : boolean;
    FLeftLineWidth : integer;
    procedure SetActive(const Value: boolean);
  protected
    { Protected declarations }
//    procedure WndProc (var Message: TMessage); override;
    procedure sDrawSelected(Item: TMenuItem; ACanvas: TCanvas; ARect: TRect);
    procedure sDrawItem(Item: TMenuItem; ACanvas: TCanvas; ARect: TRect);
    procedure sMeasureItem(Sender: TObject; ACanvas: TCanvas; var Width, Height: Integer);
    procedure sAdvancedDrawItem(Sender: TObject; ACanvas: TCanvas; ARect: TRect; State: TOwnerDrawState);
  public
    FForm: TCustomForm;
    { Public declarations }
    constructor Create (AOwner: TComponent); override;
    procedure Invalidate;
    procedure SearchMenus(iClassType: integer); // 1 - MainMenu; 2 - PopupMenu
    procedure HookMainMenu(MainMenu: TMainMenu);
  published
    { Published declarations }
    property sStyle : TsPassivePaintStyle read FsStyle write FsStyle;
    property Active: boolean read FActive write SetActive;
  end;

  TsMainMenuManager = class(TsCustomMenuManager)
  private
    { Private declarations }
  protected
    { Protected declarations }
//    procedure WndProc (var Message: TMessage); override;
  public
    { Public declarations }
    constructor Create (AOwner: TComponent); override;
    procedure Invalidate;
  published
    { Published declarations }
  end;

implementation

{ TsCustomMenuManager }

constructor TsCustomMenuManager.Create(AOwner: TComponent);
begin
  inherited;
  sStyle := TsPassivePaintStyle.Create(Self);
  FMnuList := TList.Create;
  FForm := sVclUtils.GetParentForm(AOwner);
  FLeftLineWidth := 24;
  sStyle.COC := COC_TsCustomMenuManager;
end;

procedure TsCustomMenuManager.sAdvancedDrawItem(Sender: TObject; ACanvas: TCanvas; ARect: TRect; State: TOwnerDrawState);
var
  R: TRect;
begin
  R := ARect;
  InflateRect(R, 1, 1);

  if odSelected in State then begin
    sDrawSelected(TMenuItem(Sender), ACanvas, R);
  end
  else begin
    sDrawItem(TMenuItem(Sender), ACanvas, R);
  end;
end;

procedure TsCustomMenuManager.Invalidate;
begin
//
end;

procedure TsCustomMenuManager.SearchMenus(iClassType : integer);
var
  i, j : integer;
begin
  FMnuList.Clear;
  for i := 0 to FForm.ComponentCount - 1 do begin
    case iClassType of
      1: begin
        if FForm.Components[i] is TMainMenu then begin
          FMnuList.Add(TMainMenu(FForm.Components[i]));
          HookMainMenu(TMainMenu(FForm.Components[i]));
        end;
      end
    end;
  end;
end;

procedure TsCustomMenuManager.SetActive(const Value: boolean);
var
  i : integer;
begin
  if FActive <> Value then begin
    if Value then begin
      SearchMenus(1);
      if FMnuList.Count > 0 then begin
        FActive := Value;
      end;
    end
    else begin
      for i := 0 to FMnuList.Count - 1 do begin
        TMainMenu(FMnuList.Items[i]).OwnerDraw := False;
        FMnuList.Clear;
      end;
      FActive := Value;
    end;
  end;
end;

procedure TsCustomMenuManager.HookMainMenu(MainMenu: TMainMenu);
var
  i : integer;
  procedure HookSubItems(Item: TMenuItem);
  var
    i : integer;
  begin
    for i := 0 to Item.Count - 1 do begin
      Item.Items[i].OnAdvancedDrawItem := sAdvancedDrawItem;
      Item.Items[i].OnMeasureItem := sMeasureItem;
      HookSubItems(Item.Items[i]);
    end;
  end;
begin
  for i := 0 to MainMenu.Items.Count - 1 do begin
    MainMenu.Items[i].OnAdvancedDrawItem := sAdvancedDrawItem;
    MainMenu.Items[i].OnMeasureItem := sMeasureItem;
    HookSubItems(MainMenu.Items[i]);
  end;
  MainMenu.OwnerDraw := True;
end;

procedure TsCustomMenuManager.sMeasureItem(Sender: TObject;
  ACanvas: TCanvas; var Width, Height: Integer);
begin
  Width := Width + FLeftLineWidth + sStyle.Shadow.Offset;
//
end;

procedure TsCustomMenuManager.sDrawSelected(Item: TMenuItem; ACanvas: TCanvas; ARect: TRect);
begin
  ACanvas.Brush.Color := clNavy;
  ACanvas.Rectangle(ARect);
end;

procedure TsCustomMenuManager.sDrawItem(Item: TMenuItem; ACanvas: TCanvas; ARect: TRect);
var
  DC : HDC;
  P : TPoint;
  R : Trect;
begin
//  ACanvas.Brush.Color := clMenu;
//  ACanvas.Rectangle(ARect);
//  Frame3D(ACanvas, R, sStyle.Painting.ColorBorderTop,  sStyle.Painting.ColorBorderBottom, 1);
//  Bitmap.Width := r.Right-r.Left+1;
//  Bitmap.Height := r.Bottom-r.Top+1;
  R := ARect;
  DC := GetDC(0);
//  P.x := FForm.Left;
//  P.y := FForm.Top;
  P := Point(FForm.ClientRect.Left, FForm.ClientRect.Top);
  P := FForm.ClientToScreen(P);

//  OffsetRect(R, P.x , P.y);
  BitBlt(ACanvas.Handle, R.Left, R.Top,
            R.Right - R.Left, R.Bottom - R.Top, DC, R.Left + P.x, R.Top + P.y, SrcCopy);
  ReleaseDC(0, DC);
  Frame3d(ACanvas, R, sStyle.Painting.ColorBorderTop, sStyle.Painting.ColorBorderBottom, 1);
end;

{ TsMainMenuManager }

constructor TsMainMenuManager.Create(AOwner: TComponent);
begin
  inherited;
  sStyle.COC := COC_TsMainMenuManager;
end;

procedure TsMainMenuManager.Invalidate;
begin

end;


end.
